// ListWin -- list top-level visible windows
using System;
using System.Text;
using System.Drawing;
using System.Diagnostics;
using System.Runtime.InteropServices;
using Win32API; // home-brew wrapper for native API
using WinArray; // home-brew window iterator

class MyApp {
	// global command-line switches
	static bool bRectangle = false; // show window rect using Rectangle
	static bool bRect = false;      // show window Rect using RECT
	static bool bClassName = false; // show class name
	static bool bTitle = false;	  // show title
	static bool bHwnd = false;		  // show HWND

	[STAThread]
	// main entry point
	static int Main(string[] args) {
		// Parse command line. Switches can come in any order.
		if (args.GetLength(0)<=0)
			return help();
		for (int i=0, len=args.GetLength(0); i<len; i++) {
			if (args[i].StartsWith("/") || args[i].StartsWith("-") ) {
				for (int j=1; j<args[i].Length; j++) {
					switch (args[i][j]) {
					case 'c': bClassName = true; break;
					case 'h': bHwnd = true; break;
					case 'r': bRect = true; break;
					case 'R': bRectangle = true; break;
					case 't': bTitle = true; break;
					case '?': default: return help();
					}
				}
			}
		}
		
		WindowArray itw = new WindowArray();
		foreach (int hwnd in itw) {
			if (Win32.IsWindowVisible(hwnd)) {
				if (bHwnd) {
					Console.Write("{0:x8}", hwnd);
				}
				if (bClassName) {
					StringBuilder cname = new StringBuilder(256);
					Win32.GetClassName(hwnd, cname, cname.Capacity);
					Console.Write(" {0}",cname);
				}
				if (bRectangle) {
					Rectangle rc = new Rectangle();
					Win32.GetWindowRect(hwnd, ref rc);
					Console.Write(" {0}",rc);
				} else if (bRect) {
					RECT rc = new RECT();
					Win32.GetWindowRect(hwnd, ref rc);
					Console.Write(" {0}",rc);
				}
				if (bTitle) {
					StringBuilder title = new StringBuilder(256);
					Win32.GetWindowText(hwnd, title, title.Capacity);
					Console.Write(" {0}",title);
				}
				Console.WriteLine();
			}
		}
		return 0;
	}


}
