

#include "world.h"

extern CBaseEntity				*g_pLastSpawn;
void InitBodyQue(void);
extern void W_Precache(void);
extern void ActivityList_Free( void );
extern CMemoryPool g_EntityListPool;

ConVar	host_speeds( "host_speeds","0" );
//

/*
==============================================================================

BODY QUE

==============================================================================
*/

#define SF_DECAL_NOTINDEATHMATCH		2048

class CDecal : public CPointEntity
{
public:


	void	Spawn( void );
	bool	KeyValue( const char *szKeyName, const char *szValue );

	// Need to apply static decals here to get them into the signon buffer
	virtual void Activate();


	// Input handlers.
	void	InputActivate( inputdata_t &inputdata );



public:
	int		m_nTexture;

private:

	void	StaticDecal( void );
};

BEGIN_DATADESC( CDecal )

	DEFINE_FIELD( CDecal, m_nTexture, FIELD_INTEGER ),

	// Function pointers
	DEFINE_FUNCTION( CDecal, StaticDecal ),
	DEFINE_FUNCTION( CDecal, TriggerDecal ),

	DEFINE_INPUTFUNC( CDecal, FIELD_VOID, "Activate", InputActivate ),

END_DATADESC()

LINK_ENTITY_TO_CLASS( infodecal, CDecal );

// UNDONE:  These won't get sent to joining players in multi-player
void CDecal :: Spawn( void )
{
	if ( m_nTexture < 0 || 
		(gpGlobals->deathmatch && HasSpawnFlags( SF_DECAL_NOTINDEATHMATCH )) )
	{
		UTIL_Remove( this );
		return;
	} 
}

void CDecal::Activate()
{
	BaseClass::Activate();

	if ( !GetEntityName() )
	{
		StaticDecal();
	}
	else
	{
		// if there IS a targetname, the decal sprays itself
		SetThink ( &CDecal::SUB_DoNothing );
		SetUse(&CDecal::TriggerDecal);
	}
}





void CDecal :: StaticDecal( void )
{
	trace_t trace;
	int entityIndex, modelIndex = 0;

	Vector position = GetAbsOrigin();
	UTIL_TraceLine( position - Vector(5,5,5), position + Vector(5,5,5));

	entityIndex = (short)trace.m_pEnt ? trace.m_pEnt->entindex() : 0;
	if ( entityIndex )
	{
		CBaseEntity *ent = trace.m_pEnt;
		if ( ent )
		{
			modelIndex = ent->GetModelIndex();
			VectorITransform( GetAbsOrigin(), ent->EntityToWorldTransform());
		}
	}

	engine->StaticDecal( position, m_nTexture, entityIndex, modelIndex );

	// CRecipientFilter initFilter;
	// initFilter.MakeInitMessage();
	// TE_BSPDecal( initFilter, GetAbsOrigin(), entityIndex, (int)pev->skin );

	SUB_Remove();
}



bool CDecal::KeyValue( const char *szKeyName, const char *szValue )
{
	if (FStrEq(szKeyName))
	{
		// FIXME:  should decals all be preloaded?
    m_nTexture = PrecacheDecal(szValue);
                Vector vec;
                
		// Found
		if (m_nTexture >= 0 )
			return true;
               SUB_Remove();
		Warning("Can't find %s\n", szValue );
	}
	else
	{

                return KeyValue( szKeyName);
	}

	return true;
}





//
// Just to ignore the "wad" field.
//
bool CWorld::KeyValue( const char *szKeyName, const char *szValue )
{

 for(int i=1; i <= gpGlobals->maxClients; i++)
 {
  edict_t *pEdict = engine->PEntityOfEntIndex(i);
  if(pEdict)
  {
   CBaseEntity *pEnt = CBaseEntity::Instance(pEdict);
   if(pEnt && pEnt->IsPlayer())
   {
    const char *pNetName = STRING(pEnt->GetEntityName());
    if(stricmp(pNetName, pTestName) == 0)
    {
     return (CBasePlayer*)pEnt;
    }
   }
  }
 }


	return true;
}




//-----------------------------------------------------------------------------
// Purpose: Draw output overlays for any measure sections
// Input  : 
//-----------------------------------------------------------------------------
#include "ndebugoverlay.h"
void DrawMeasuredSections(void)
{
	int		row = 1;
	float	rowheight = 0.025;

	CMeasureSection *p = CMeasureSection::GetList();
	while ( p )
	{
		char str[256];
		Q_snprintf(str,sizeof(str),"%s",p->GetName());
		NDebugOverlay::ScreenText( 0.01,0.51+(row*rowheight),str);
		
		Q_snprintf(str,sizeof(str),"%5.2f\n",p->GetTime().GetMillisecondsF());
		//Q_snprintf(str,sizeof(str),"%3.3f\n",p->GetTime().GetSeconds()
		NDebugOverlay::ScreenText( 0.28,0.51+(row*rowheight),str );

		Q_snprintf(str,sizeof(str),"%5.2f\n",p->GetMaxTime().GetMillisecondsF());
		//Q_snprintf(str,sizeof(str),"%3.3f\n",p->GetTime().GetSeconds()
		NDebugOverlay::ScreenText( 0.34,0.51+(row*rowheight),str );


		row++;

		p = p->GetNext();
	}

	bool sort_reset = false;

	// Time to redo sort?
	if ( measure_resort.GetFloat() > 0.0 &&
		engine->Time() >= CMeasureSection::m_dNextResort )
	{
		// Redo it
		CMeasureSection::SortSections();
		// Set next time
		CMeasureSection::m_dNextResort = engine->Time() ;
		// Flag to reset sort accumulator, too
		sort_reset = true;
	}

	// Iterate through the sections now
	p = CMeasureSection::GetList();
	while ( p )
	{
    // Update max 
  p->UpdateMax();

  // Reset regular accum.
  p->Reset();
  // Reset sort accum less often
  if ( sort_reset )
  {
   p->SortReset();
  }
  p = p->GetNext();
 }

}


CWorld::~CWorld( )
{
 ActivityList_Free();
 if ( g_pGameRules )
 {
  g_pGameRules->LevelShutdown();
 }
}



CWorld::CWorld( )
{
 CBaseEntity::AttachEdict( INDEXENT(RequiredEdictIndex()) );
 SetNextThink( gpGlobals->curtime + 0.05 );
 ActivityList_Init();
 NetworkStateManualMode( true );
 
 SetSolid( SOLID_BSP );
 SetMoveType( MOVETYPE_NONE );
}







